/****************************************************************************
 *
 * Project: Generic test routines.
 *
 * Copyright by Olimex Ltd. All rights reserved.
 *
 * File: extcon.h
 * Description: External connector test configuration for LPC-P2919
 * Developer: Dimitar Dimitrov ( dinuxbg,gmail.com )
 *
 * Last change: $Date: 2011/03/31 $
 * Revision: $Revision: #2 $
 * Id: $Id: //depot/IAR/auxiliary_arm2_5_xx/TestCode/ST/STM32/STM32F217x/Project/Test/extcon-conf.c#2 $
 *
 ****************************************************************************/

#include <stdint.h>
#include <stdbool.h>
#include <stdio.h>
#include <assert.h>
#include <stm32f4xx_gpio.h>

#include "extcon.h"


#define ARRAY_NUMELEM(A)	(sizeof((A))/sizeof((A)[0]))


#define PIN(CON_PIN, PRT, PRT_PIN, CHIP_PIN)\
{ \
  .con_pin = CON_PIN, \
  .port_pin = ((PRT)<<8) | (PRT_PIN), \
  .chip_pin = CHIP_PIN, \
}

static void print_port_pin(char *buf, const struct extcon_pin *pin)
{
  sprintf(buf, "P%c[%d]",pin->port_pin>>8, pin->port_pin & 0xff);
}

static int stm32f2xx_gpio_read(const struct extcon_pin *pin)
{
GPIO_TypeDef* GPIO;

  switch(pin->port_pin>>8)
  {
    case 'A':
      GPIO = GPIOA;
    break;
    case 'B':
      GPIO = GPIOB;
    break;
    case 'C':
      GPIO = GPIOC;
    break;
    case 'D':
      GPIO = GPIOD;
    break;
  }

  return GPIO_ReadInputDataBit(GPIO, 1<<(pin->port_pin & 0xff));
}


static void stm32f2xx_gpio_set_inp(const struct extcon_pin *pin)
{
GPIO_TypeDef* GPIO;
GPIO_InitTypeDef GPIO_InitStructure;

  GPIO_InitStructure.GPIO_Pin = 1<<(pin->port_pin & 0xff);
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

  switch(pin->port_pin>>8)
  {
    case 'A':
      GPIO = GPIOA;
    break;
    case 'B':
      GPIO = GPIOB;
    break;
    case 'C':
      GPIO = GPIOC;
    break;
    case 'D':
      GPIO = GPIOD;
    break;
  }

  GPIO_Init(GPIO, &GPIO_InitStructure);
}


static void stm32f2xx_gpio_set_outp(const struct extcon_pin *pin, int val)
{
GPIO_TypeDef* GPIO;
GPIO_InitTypeDef GPIO_InitStructure;

  GPIO_InitStructure.GPIO_Pin = 1<<(pin->port_pin & 0xff);
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

  switch(pin->port_pin>>8)
  {
    case 'A':
      GPIO = GPIOA;
    break;
    case 'B':
      GPIO = GPIOB;
    break;
    case 'C':
      GPIO = GPIOC;
    break;
    case 'D':
      GPIO = GPIOD;
    break;
  }
  GPIO_Init(GPIO, &GPIO_InitStructure);
  GPIO_WriteBit(GPIO,1<<(pin->port_pin & 0xff), (BitAction)val);
}


static const struct extcon_pin uextcon_main_outp = PIN(9, 'A',5, 21);

static const struct extcon_pin uextcon_pins[] = 
{
  /* CONN_PIN     PORT_PIN    CHIP_PIN */
  PIN( 3,	  'A',  9, 	42),
  PIN( 4,	  'A',  10, 	43),
  PIN( 5,	  'B',  6, 	58),
  PIN( 6,	  'B',  7, 	59),
  PIN( 7,	  'A',  6, 	22),
  PIN( 8,	  'A',  7, 	23),
//  PIN( 9,	  'A',  5, 	21),
  PIN( 10,	  'A',  4, 	20)
};


static const struct extcon_pin ext_main_outp = 	PIN(32,	'D', 2, 54);

static const struct extcon_pin ext_pins[] = 
{
  /* CONN_PIN     PORT_PIN    CHIP_PIN */
  //PIN( 1,	  '',  , 	),    // 3.3V
  // A
  PIN( 2,	  'A',  1, 	15),
  PIN( 3,	  'A',  8, 	41),
  
  // B
  PIN( 4,	  'B',  0, 	26),
  PIN( 5,	  'B',  1, 	27),
  //PIN( 6,	  'B',  2, 	28),  // Boot jumper
  PIN( 7,	  'B',  5, 	57),
  //PIN( 8,	  'B',  8, 	61),  // CAN RX
  //PIN( 9,	  'B',  9, 	62),  // CAN TX
  //PIN( 10,	  '',  , 	),    // VDDA
  //PIN( 11,	  '',  , 	),    // GNDA
  PIN( 12,	  'B',  10, 	29),
  PIN( 13,	  'B',  11, 	30),
  PIN( 14,	  'B',  12, 	33),
  PIN( 15,	  'B',  13, 	34),
  PIN( 16,	  'B',  14, 	34),
  PIN( 17,	  'B',  15, 	36),
  
  //PIN( 18,	  '',  , 	),    // Reset
  
  // C
  PIN( 19,	  'C',  0, 	8),
  PIN( 20,	  'C',  1, 	9),
  PIN( 21,	  'C',  2, 	10),
  PIN( 22,	  'C',  3, 	11),
  PIN( 23,	  'C',  4, 	24),
  PIN( 24,	  'C',  5, 	25),
  PIN( 25,	  'C',  6, 	37),
  PIN( 26,	  'C',  7, 	38),
  PIN( 27,	  'C',  8, 	39),
  PIN( 28,	  'C',  9, 	40),
  PIN( 29,	  'C',  10, 	51),
  //PIN( 30,	  'C',  12, 	53),  // LED
  PIN( 31,	  'C',  13, 	2),
  
  // D
  //PIN( 32,	  'D',  2, 	54),  // Pull-pin
  
  //PIN( 33,	  '',  , 	),    // 5V USV
  //PIN( 34,	  '',  , 	),    // VIN
  //PIN( 35,	  '',  , 	),    // GND
};

static const struct extcon_jack extcons[] =
{
  {
    .name = "UEXT",
    .main_outp_pin = &uextcon_main_outp,
    .pins = uextcon_pins,
    .num_pins = ARRAY_NUMELEM(uextcon_pins)
  },
  {
    .name = "ext",
    .main_outp_pin = &ext_main_outp,
    .pins = ext_pins,
    .num_pins = ARRAY_NUMELEM(ext_pins)
  },
};

static void setup_delay(void)
{
  for(volatile int i=0; i<10000; i++);
}

const struct extcon_board extcon_board_def = {
  .name = "STM32-P405",
  .connectors = extcons,
  .num_connectors = ARRAY_NUMELEM(extcons),

  .gpio_read = stm32f2xx_gpio_read,
  .gpio_set_inp = stm32f2xx_gpio_set_inp,
  .gpio_set_outp = stm32f2xx_gpio_set_outp,

  .print_port_pin = print_port_pin,
  .print_con_pin = 0,	/* use default implementation */
  .print_chip_pin = 0,	/* use default implementation */

  .setup_delay = setup_delay,

  .puts = 0,
};



